from mercurius.strategy.base import expert
from mercurius.strategy import tools
import numpy as np
import logging

class ubah(expert):
    '''Uniform Buy and Hold'''
    def __init__(self):
        super(ubah, self).__init__()


    def get_b(self, data, last_b):
        return last_b

if __name__ == '__main__':
    tools.run(ubah(),plot=True)



from mercurius.strategy.base import expert
from mercurius.strategy import tools
import numpy as np
import logging

class anticor(expert):
    '''
    anti-correlation olps
    '''
    def __init__(self, window=30):
        super(anticor, self).__init__()
        self.window = window

    def get_b(self, data, last_b):
        for k in np.arange(1,self.window):
            self.exp_w[k-1,:] = self.update(data, self.exp_w[k-1,:].T, k)

        numerator = 0
        denominator = 0

        for k in np.arange(1,self.window):
            numerator += self.exp_ret[k-1, 0] * self.exp_w[k-1,:]
            denominator += self.exp_ret[k-1,0]

        weight = numerator.T / denominator

        return weight

    def update(self, data,last_b, w):
        T, N = data.shape
        b = last_b

        if T >= 2*w:
            LX1 = np.log(data[T-2*w+1:T-w,:]) #(k,N)
            LX2 = np.log(data[T-w+1:T,:])

            mu1 = np.mean(LX1, axis=0)
            sig1 = np.std(LX1, axis=0, ddof=1)
            mu2 = np.mean(LX2, axis=0)
            sig2 = np.std(LX2, axis=0, ddof=1)
            sigma = np.dot(sig1.T,sig2)

            mu_matrix = np.ones((mu2.shape[0],mu2.shape[0]), dtype = bool)
            for i in range(0, mu2.shape[0]):
                for j in range(0, mu2.shape[0]):
                    if mu2[i] > mu2[j]:
                        mu_matrix[i,j] = True
                    else:
                        mu_matrix[i,j] = False

            mCov = (1.0/np.float64(w-1)) * np.dot(np.transpose(np.subtract(LX1,mu1)),np.subtract(LX2,mu2))

            mCorr = np.where(sigma!=0, np.divide(mCov,sigma), sigma) #set

#Multiply the correlation matrix by the boolean matrix comparing mu2[i] to mu2[j] and by the boolean matrix where correlation matrix is greater than zero

            claim = mCorr * np.multiply(mCorr>0,mu_matrix)

#The boolean claim matrix will be used to obtain only the entries that meet the criteria that mu2[i] > mu2[j] and mCorr is > 0 for the i_corr and j_corr matrices

            bool_claim = claim > 0

#If stock i is negatively correlated with itself we want to add that correlation to all instances of i.  To do this, we multiply a matrix of ones by the diagonal of the correlation matrix row wise.

            i_corr = np.ones((mu1.shape[0],mu2.shape[0])) * np.diagonal(mCorr)[:,None]

#Since our condition is when the correlation is negative, we zero out any positive values, also we want to multiply by the bool_claim matrix to obtain valid entries only

            i_corr = np.where(i_corr > 0,0,i_corr)
            i_corr = i_corr * bool_claim

#Subtracting out these negative correlations is essentially the same as adding them to the claims matrix

            claim -= i_corr

#We repeat the same process for stock j except this time we will multiply the diagonal of the correlation matrix column wise

            j_corr = np.ones((mu1.shape[0],mu2.shape[0])) * np.diagonal(mCorr)[None,:]

#Since our condition is when the correlation is negative, we zero out any positive values again multiplying by the bool_claim matrix

            j_corr = np.where(j_corr > 0,0,j_corr)
            j_corr = j_corr * bool_claim

#Once again subtract these to obtain our final claims matrix
            claim -= j_corr

            sum_claim = np.sum(claim, axis=1) #(N,)
#Then divide each element of the claims matrix by the sum of it's corresponding row
            transfer = claim / sum_claim[:,None] #(N,N)
#Multiply the original weights to get the transfer matrix row wise
            transfer = np.multiply(transfer,b[:,None])
#Replace the nan with zeros in case the divide encountered any
            transfer = np.where(np.isnan(transfer),0,transfer)
#We don't transfer any stock to itself, so we zero out the diagonals
            #np.fill_diagonal(transfer,0) #does not influence result at all
#Create the new portfolio weight adjustments, by adding the j direction weights or the sum by columns and subtracting the i direction weights or the sum by rows
            adjustment = np.sum(transfer, axis=0) - np.sum(transfer,axis=1) #transfer_ij
            #print adjustment
            b += adjustment
        return b

    def trade(self, data, tc=0, min_period=1):
        '''Meta Trader
        :param data:
        :param tc: transaction cost, default 0
        :param min_period: minimum period to start trading, default 1
        '''
        logging.info('------------------------------------------\n')
        logging.info('Parameters [tc: %f].\n' % tc)
        logging.info('day\t Daily Return\t Total Return\n')
        logging.info('------------------------------------------\n')

        n, m = data.shape

        cum_ret = 1 #cumulative return aka total return
        cumprod_ret = np.ones((n,1), np.float)
        daily_ret = np.ones((n, 1), np.float) #daily return

        b = np.ones((m,1), np.float) / m # initialize b
        re_b = np.zeros((m,1), np.float) # initialize rebalanced b
        daily_portfolio = np.zeros((n,m))

        self.exp_ret = np.ones((self.window-1,1))
        self.exp_w = np.ones((self.window-1,m)) / m

        for t in range(n):
            if t>= min_period:
                b = self.get_b(data[:t,:], re_b)

            # double check (Normalize the constraint)
            b = b / np.sum(b)
            daily_portfolio[t,:] = b.reshape((1,m))
            daily_ret[t] = np.dot(data[t,:],b)*(1-tc/2*np.sum(np.absolute(b-re_b)))
            cum_ret = cum_ret * daily_ret[t]
            cumprod_ret[t] = cum_ret

            re_b = b * data[t,:][:,None] / daily_ret[t]

            for k in np.arange(1,self.window):
                self.exp_ret[k-1,0] = np.dot(self.exp_ret[k-1,0] * data[k,:],self.exp_w[k-1,:].T)
            self.exp_ret[:,0] /= np.sum(self.exp_ret[:,0])

            logging.info('%d\t%f\t%f\n' % (t+1, daily_ret[t], cumprod_ret[t]))

        logging.info('tc=%f, Final Return: %.2f\n' % (tc, cum_ret))

        self.pDiff = daily_ret

if __name__ == "__main__":
    tools.run(anticor())



from mercurius.strategy.base import expert
from mercurius.strategy import tools
import numpy as np

class m0(expert):
    def __init__(self, beta=0.5, C=None):
        super(m0, self).__init__()
        self.beta = beta
        self.C = C

    def get_b(self, data, last_b):
        m = data.shape[1]
        if self.C is None:
            self.C = np.zeros((m,1))
        b = (self.C+self.beta)/(m*self.beta+np.ones((1,m)).dot(self.C))
        max_ind = np.argmax(data[-1,:])
        self.C[max_ind] += 1
        return b

if __name__ == "__main__":
    tools.run(m0())



from mercurius.strategy.base import expert
from mercurius.strategy import tools
import numpy as np
import logging

class cornk(expert):
    '''
    Correlation driven non parametric Uniform
    '''
    def __init__(self, K=5, L=1, pc=0.1, exp_w=None):
        '''
        :param K: maximum window size
        :param L: splits into L parts, in each K
        '''
        super(cornk, self).__init__()
        self.K = K
        self.L = L
        self.pc = pc
        self.exp_ret = np.ones((K,L))
        self.exp_w = exp_w


    def get_b(self, data, last_b):

        n, m = data.shape

        if self.exp_w is None:
            self.exp_w = np.ones((self.K*self.L,m))/m

        for k in np.arange(self.K):
            for l in np.arange(self.L):
                rho = (l-1)/self.L
                self.exp_w[(k-1)*self.L+l,:] = self.update(data, k, rho)

        nc = np.ceil(self.pc*self.K*self.L)
        exp_ret_sort = np.sort(self.exp_ret.ravel())
        ret_rho = exp_ret_sort[int(self.K*self.L-nc)]
        numerator = 0
        denominator = 0

        for k in np.arange(self.K):
            for l in np.arange(self.L):
                p = 1 if self.exp_ret[k,l] >= ret_rho else 0
                numerator += p * self.exp_ret[k,l] * self.exp_w[k*self.L,:]
                denominator += p * self.exp_ret[k,l]

        b = numerator.T / denominator
        return b

    def update(self, data, w, c):
        '''
        :param w: window sze
        :param c: correlation coefficient threshold
        '''
        T, N = data.shape
        m = 0
        histdata = np.zeros((T,N))

        if T <= w+1:
            '''use uniform portfolio weight vector'''
            return np.ones(N) / N

        if w==0:
            histdata = data[:T,:]
            m = T
        else:
            for i in np.arange(w, T):
                d1 = data[i-w:i,:].ravel()
                d2 = data[T-w:T,:].ravel()
                datacorr = np.corrcoef(d1,d2)[0,1]

                if datacorr >= c:
                    m += 1
                    histdata[m,:] = data[i-1,:] #minus one to avoid out of bounds issue

        if m==0:
            return np.ones(N) / N

        b = tools.opt_weights(histdata)
        return b


    def trade(self, data, tc=0, min_period=1):
        '''Meta Trader
        :param data:
        :param tc: transaction cost, default 0
        :param min_period: minimum period to start trading, default 1
        '''
        logging.info('------------------------------------------\n')
        logging.info('Parameters [tc: %f].\n' % tc)
        logging.info('day\t Daily Return\t Total Return\n')
        logging.info('------------------------------------------\n')

        n, m = data.shape

        cum_ret = 1 #cumulative return aka total return
        cumprod_ret = np.ones((n,1), np.float)
        daily_ret = np.ones((n, 1), np.float) #daily return

        b = np.ones((m,1), np.float) / m # initialize b
        re_b = np.zeros((m,1), np.float) # initialize rebalanced b
        daily_portfolio = np.zeros((n,m))

        self.exp_ret = np.ones((self.K,self.L))
        self.exp_w = np.ones((self.K*self.L,m)) / m

        for t in range(n):
            if t>= min_period:
                b = self.get_b(data[:t,:], re_b)

            # double check (Normalize the constraint)
            b = b / np.sum(b)
            daily_portfolio[t,:] = b.reshape((1,m))
            daily_ret[t] = np.dot(data[t,:],b)*(1-tc/2*np.sum(np.absolute(b-re_b)))
            cum_ret = cum_ret * daily_ret[t]
            cumprod_ret[t] = cum_ret

            re_b = b * data[t,:][:,None] / daily_ret[t]

            for k in np.arange(self.K):
                for l in np.arange(self.L):
                    self.exp_ret[k,l] *= data[t,:].dot(self.exp_w[(k-1)*self.L+l,:].T)

            logging.info('%d\t%f\t%f\n' % (t+1, daily_ret[t], cumprod_ret[t]))

        logging.info('tc=%f, Final Return: %.2f\n' % (tc, cum_ret))

if __name__ == "__main__":
    tools.run(cornk())



from mercurius.strategy.base import expert
from mercurius.strategy import tools
import numpy as np
import logging

class up(expert):
    """
    Universal Portfolio

    T. M. Cover
    """
    def __init__(self):
        super(up, self).__init__()


    def get_b(self, data, last_b):
        del0 = 4e-3
        delta = 5e-3
        M = int(1e1)
        S = int(0.5e1)

        N = data.shape[1]

        r = np.ones((N,1)) / N
        b = np.ones(r.shape)

        allM = np.zeros((N,M))

        for m in range(M):
            b = r
            for i in range(S):
                bnew = np.copy(b)
                j = np.random.randint(low=1, high=N)
                a = np.random.randint(-1,1)
                if a == 0:
                    a = 1
                bnew[j] = b[j] + (a*delta)
                if bnew[j] >= del0 and bnew[N-1] >= del0:
                    x = self.findQ(b, data, del0, delta)
                    y = self.findQ(bnew, data, del0, delta)
                    pr = np.minimum(y/x, 1)
                    temp = np.random.uniform(0,1)
                    if temp < pr:
                        b = bnew

            allM[:,m] = b.conj().T

        return np.sum(allM, axis=1) / M

    def findQ(self, b, data, del0, delta):
        N = data.shape[1]
        P = np.prod(data.dot(b))
        Q = np.dot(P, np.minimum(1, np.exp((b[N-1]-(2*del0))/((N-1)*delta))))
        return Q

if __name__ == '__main__':
    tools.run(up())



from mercurius.strategy.base import expert
from mercurius.strategy import tools
import numpy as np
import pandas as pd
import scipy.stats
from numpy.linalg import inv
from numpy import diag, sqrt, log, trace
import logging
from cwmr_std import cwmr


class cwmr_var(cwmr):
    def __init__(self, eps=-0.5, confidence=0.95, b=None):
        """
        :param eps: Mean reversion threshold (expected return on current day must be lower
                    than this threshold). Recommended value is -0.5.
        :param confidence: Confidence parameter for profitable mean reversion portfolio.
                    Recommended value is 0.95.
        """
        super(cwmr_var, self).__init__()
        # input check
        if not (0 <= confidence <= 1):
            raise ValueError('confidence must be from interval [0,1]')

        self.eps = eps
        self.theta = scipy.stats.norm.ppf(confidence)
        self.b = b


    def init_weights(self, m):
        return np.ones(m) / m


    def init_step(self, X):
        m = X.shape[1]
        self.sigma = np.matrix(np.eye(m) / m**2)


    def get_b(self, data, last_b):
        # initialize
        x = data[-1,:]
        x = np.reshape(x, (1, x.size))
        if self.b is None:
            self.b = np.ones(x.size) / x.size
        last_b = self.b
        last_b = np.reshape(last_b, (1, last_b.size))
        self.init_step(x)
        m = len(x)
        mu = np.matrix(last_b).T
        sigma = self.sigma
        theta = self.theta
        eps = self.eps
        x = np.matrix(x).T    # matrices are easier to manipulate

        # 4. Calculate the following variables
        M = mu.T * x
        V = x.T * sigma * x
        x_upper = sum(diag(sigma) * x) / trace(sigma)

        # 5. Update the portfolio distribution
        mu, sigma = self.update(x, x_upper, mu, sigma, M, V, theta, eps)

        # 6. Normalize mu and sigma
        mu = tools.simplex_proj2(mu)
        sigma = sigma / (m**2 * trace(sigma))
        """
        sigma(sigma < 1e-4*eye(m)) = 1e-4;
        """
        self.sigma = sigma
        return mu

    def update(self, x, x_upper, mu, sigma, M, V, theta, eps):
        # lambda from equation 7
        foo = (V - x_upper * x.T * np.sum(sigma, axis=1)) / M**2
        a = 2 * theta * V * foo
        b = foo + 2 * theta * V * (eps - log(M))
        c = eps - log(M) - theta * V

        a,b,c = a[0,0], b[0,0], c[0,0]

        lam = max(0,
                  (-b + sqrt(b**2 - 4 * a * c)) / (2. * a),
                  (-b - sqrt(b**2 - 4 * a * c)) / (2. * a))
        # bound it due to numerical problems
        lam = min(lam, 1E+7)

        # update mu and sigma
        mu = mu - lam * sigma * (x - x_upper) / M
        sigma = inv(inv(sigma) + 2 * lam * theta * diag(x)**2)
        """
        tmp_sigma = inv(inv(sigma) + theta*lam/U_sqroot*diag(xt)^2);
        % Don't update sigma if results are badly scaled.
        if all(~isnan(tmp_sigma(:)) & ~isinf(tmp_sigma(:)))
            sigma = tmp_sigma;
        end
        """
        return mu, sigma
    def trade(self, data, tc=0, min_period=1):
        '''Meta Trader
        :param data:
        :param tc: transaction cost, default 0
        :param min_period: minimum period to start trading, default 1
        '''
        logging.info('------------------------------------------\n')
        logging.info('Parameters [tc: %f].\n' % tc)
        logging.info('day\t Daily Return\t Total Return\n')
        logging.info('------------------------------------------\n')

        n, m = data.shape

        cum_ret = 1 #cumulative return aka total return
        cumprod_ret = np.ones((n,1), np.float)
        daily_ret = np.ones((n, 1), np.float) #daily return

        b = np.ones((m,1), np.float) / m # initialize b
        re_b = np.zeros((m,1), np.float) # initialize rebalanced b
        daily_portfolio = np.zeros((n,m))

        for t in range(n):
            if t>= min_period:
                b = self.get_b(data[:t,:], re_b)

            # double check (Normalize the constraint)
            b = b / np.sum(b)
            daily_portfolio[t,:] = b.reshape((1,m))
            daily_ret[t] = np.dot(data[t,:],b)*(1-tc/2*np.sum(np.absolute(b-re_b)))
            cum_ret *= daily_ret[t]
            cumprod_ret[t] = cum_ret
            re_b = b * np.reshape(data[t,:], (1,m)) / daily_ret[t]

            logging.info('%d\t%f\t%f\n' % (t+1, daily_ret[t], cumprod_ret[t]))

        logging.info('tc=%f, Final Return: %.2f\n' % (tc, cum_ret))

        self.pDiff = daily_ret

if __name__ == '__main__':
    tools.run(cwmr_var())



import numpy as np
from time import time
from sys import exit
import logging
import matplotlib as mpl
mpl.use('agg')
import matplotlib.pyplot as plt

class expert(object):
    '''Meta class for trading algorithms
        according to run in OLPS

    '''
    def __init__(self, history=None, last_b=None, pDiff=None):
        '''init.
        :param hisotry: record history of relative price vector if True
        :param last_b: last portfolio weight
        :difference of portfolio: p(t)/p(t-1)
        '''
        self.history = history
        self.last_b = last_b
        self.pDiff = pDiff

    def get_b(self, data, last_b):
        '''get next portfolio weight vector.
        :param data: input matrix without the lastest price
        '''
        raise NotImplementedError('subclass must implement this!')

    def trade(self, data, tc=0, min_period=1):
        '''Meta Trader
        :param data:
        :param tc: transaction cost, default 0
        :param min_period: minimum period to start trading, default 1
        '''
        logging.info('------------------------------------------\n')
        logging.info('Parameters [tc: %f].\n' % tc)
        logging.info('day\t Daily Return\t Total Return\n')
        logging.info('------------------------------------------\n')

        n, m = data.shape

        cum_ret = 1 #cumulative return aka total return
        cumprod_ret = np.ones((n,1), np.float)
        daily_ret = np.ones((n, 1), np.float) #daily return

        b = np.ones((m,1), np.float) / m # initialize b
        re_b = np.zeros((m,1), np.float) # initialize rebalanced b
        daily_portfolio = np.zeros((n,m))

        for t in range(n):
            if t>= min_period:
                b = self.get_b(data[:t,:], re_b)

            # double check (Normalize the constraint)
            b = b / np.sum(b)
            daily_portfolio[t,:] = b.reshape((1,m))
            #print(b)
            #print(data[t,:])
            daily_ret[t] = np.dot(data[t,:],b)*(1-tc/2*np.sum(np.absolute(b-re_b)))
            #print(daily_ret[t])
            cum_ret *= daily_ret[t]
            #print(cum_ret)
            if cum_ret == 0 or cum_ret > 10:
                print('too small or too large, quitting...')
                exit()
            cumprod_ret[t] = cum_ret
            re_b = b * data[t,:][:,None] / daily_ret[t]

            logging.info('%d\t%f\t%f\n' % (t+1, daily_ret[t], cumprod_ret[t]))

        logging.info('tc=%f, Final Return: %.2f\n' % (tc, cum_ret))

        self.pDiff = daily_ret
        self.last_b = np.squeeze(b)

    def finish(self, re=True, plot=False):
        """
        :param re: if return the result
        :param plot: if plot the result
        """
        if re:
            result = {}
            result['portfolio'] = np.cumprod(self.pDiff)
            result['portfolio_diff'] = self.pDiff
            result['last_b'] = self.last_b
            return result

        if plot:
            name = self.__class__.__name__
            plt.plot(result['portfolio'], label=name)
            plt.title("Portfolio")
            plt.legend(name, fancybox=True)
            plt.xlabel('period')
            plt.ylabel("portfolio")
            plt.show()
            plt.savefig('{name}.png'.format(name=name))




import numpy as np
import pandas as pd
#from nntrader.tdagent.algorithms.olmar import OLMAR
from mercurius.strategy.olmar import olmar
import tools
import logging
import portfolioopt as pfopt

class rmr2(olmar):
    def __init__(self, window=5, eps=10, tau=1e-3):
        super(rmr2, self).__init__(window, eps)
        self.tau = tau

    def get_b(self, data, last_b):
        x = data
        x_mean = np.mean(x)
        lam = max(0., (self.eps - np.dot(last_b, x)) / norm(x - x_mean) **2)
        lam = min(1e+5, lam)
        b = last_b + lam * (x - x_mean)
        b = np.log(1. + np.exp(b))
        b = b / b.sum()
        b = pfopt.truncate_weights(b)
        return b

    def predict_rmr(self, x, history):
        y = history.mean()
        y_last = None
        while y_last is None or norm(y-y_last)/norm(y_last)>self.tau:
            y_last=y
            d=norm(history-y)
            y = history.div(d, axis=0).sum() / (1./d).sum()
        return y/x

    def get_close(self, data):
        return np.cumprod(data, axis=0)

    def trade(self, data, tc=0, min_period=1):
        n, m = data.shape
        history = self.get_close(data)
        x = history[-1]

        cum_ret = 1
        cumprod_ret = np.ones((n, 1), np.float)
        daily_ret = np.ones((n, 1), np.float)

        b = np.ones((m,1), np.float) / m
        re_b = np.zeros((m,1), np.float)
        daily_portfolio = np.zeros((n, m))

        for t in range(n):
            b = self.update(x, history)
            daily_portfolio[t,:] = b.reshape((1,m))
            daily_ret[t] = np.dot(data[t,:],b)*(1-tc/2*np.sum(np.absolute(b-re_b)))
            cum_ret *= daily_ret[t]
            cumprod_ret[t] = cum_ret
            re_b = b * data[t,:][:,None] / daily_ret[t]

            logging.info('%d\t%f\t%f\n' % (t+1, daily_ret[t], cumprod_ret[t]))

        logging.info('tc=%f, Final Return: %.2f\n' % (tc, cum_ret))

        self.pDiff = daily_ret

def norm(x):
    if len(x.shape) == 1:
        axis=0
    else:
        axis=1
    return np.sqrt((x**2).sum(axis=axis))

if __name__ == "__main__":
    tools.run(rmr2())



from mercurius.strategy.base import expert
from mercurius.strategy import tools
import numpy as np
import logging

class sp(expert):
    '''Switch Portfolio'''
    def __init__(self, gamma=0.25):
        super(sp, self).__init__()
        self.gamma = gamma

    def get_b(self, data, last_b):
        T, N = data.shape
        b = last_b * (1-self.gamma-self.gamma/(N-1)) + self.gamma/(N-1)
        b = b / np.sum(b)
        return b

if __name__ == "__main__":
    tools.run(sp())



from mercurius.strategy.base import expert
from mercurius.strategy import tools
import numpy as np
import logging

class olmar(expert):
    """ On-Line Portfolio Selection with Moving Average Reversion

    Reference:
        B. Li and S. C. H. Hoi.
        On-line portfolio selection with moving average reversion, 2012.
        http://icml.cc/2012/papers/168.pdf
    """

    def __init__(self, window=5, eps=10, b=None):
        """
        :param window: Lookback window.
        :param eps(epsilon): Constraint on return for new weights on last price (average of prices).
            x * w >= eps for new weights w.
        """
        super(olmar, self).__init__()
        if window < 2:
            raise ValueError('window parameter must be >=3')
        if eps < 1:
            raise ValueError('epsilon parameter must be >=1')

        self.window = window
        self.eps = eps
        self.b = b

    def get_b(self, data, last_b):
        T, N = data.shape
        if self.b is None:
            self.b = np.ones(N) / N
        if T < self.window + 1 :
            data_phi=data[T-1,:]
        else:
            data_phi = np.zeros((1,N))
            tmp_x = np.ones((1,N))
            for i in range(self.window):
                data_phi += 1. /  tmp_x
                tmp_x *= data[T-i-1,:]
            data_phi *=  np.float32(1./self.window)

        ell = np.maximum(0, self.eps - data_phi.dot(last_b))
        #print data_phi
        x_bar = np.mean(data, dtype=np.float32)
        dd = data_phi - x_bar
        denominator = np.linalg.norm(dd) ** 2
        if denominator != 0:
            lam = ell /  denominator
        else:
            lam = 0
        self.b = self.b.ravel() + lam * dd
        self.b = tools.simplex_proj2(self.b.ravel())
        self.b = self.b[:,None]
        return self.b

if __name__ == "__main__":
    tools.run(olmar(), min_period=2)



import scipy.io as sio
import os
import sys
import logging
from time import time
import numpy as np
import matplotlib as mpl
mpl.use('agg')
import matplotlib.pyplot as plt
from scipy.optimize import minimize
from scipy.spatial.distance import cdist, euclidean

def run(algo, data=None, tc=0, min_period=1, plot=False):
    '''Run an algorithm on data with transaction fee
    :param algo: algorithm
    :param data: input data (relative price)
    :param tc: transaction fee
    '''
    logging.basicConfig(level=logging.INFO)
    try:
        if data is None:
            #data = random_portfolio()
            data = load_mat('djia')
        st = time() #start time
        algo.trade(data, tc=tc, min_period=min_period)
        logging.info("The trading time is %s" % (time()-st))

    finally:
        algo.finish(plot=plot)

def plot_result(result):
    pass

def load_mat(name):
    """ Load .mat from /data directory. """
    mod = sys.modules[__name__]
    filename = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data', name + '.mat')
    fi = sio.loadmat(filename)
    return fi['data']

def random_portfolio():
    pass

def simplex_proj(v, s=1):
    """Compute the Euclidean projection on a positive simplex
    :param v: n-dimensional vector to project
    :param s: radius of the simplex

    Reference: J. Duchi
    """
    assert s > 0, "Radius s must be positive (%d <= 0)" % s

    n, = v.shape
    #check if already on the simplex
    if v.sum() == s and np.alltrue(v>=0):
        return v

    # get the array of cumulative sums of a sorted copy of v
    u = np.sort(v)[::-1]
    cssv = np.cumsum(u)
    # get the number of >0 components of the optimal solution
    rho = np.nonzero(u*np.arange(1,n+1)>(cssv-s))[0][-1]
    #compute the Lagrange multiplier associated to the simplex constraint
    theta = (cssv[rho]-s) / (rho + 1.)
    w = (v-theta).clip(min=0)
    return w

def simplex_proj2(y):
    m = len(y)
    bget = False

    s = sorted(y, reverse=True)
    tmpsum = 0.

    for i in range(m-1):
        tmpsum += s[i]
        tmax = (tmpsum - 1) / (i + 1)
        if tmax >= s[i+1]:
            bget = True
            break

    if not bget:
        tmax = (tmpsum + s[m-1] - 1) / m
    return np.maximum(0, y-tmax)

def l1_median_VaZh(X, eps=1e-5):
    '''calculate the L1_median of X with the l1median_VaZh method'''
    y = np.mean(X, 0)

    while True:
        D = cdist(X, [y])
        nonzeros = (D != 0)[:, 0]

        Dinv = 1 / D[nonzeros]
        Dinvs = np.sum(Dinv)
        W = Dinv / Dinvs
        T = np.sum(W * X[nonzeros], 0)
        num_zeros = len(X) - np.sum(nonzeros)
        if num_zeros == 0:
            y1 = T
        elif num_zeros == len(X):
            return y
        else:
            R = (T - y) * Dinvs
            r = np.linalg.norm(R)
            rinv = 0 if r==0 else num_zeros/r
            y1 = max(0, 1-rinv)*T + min(1, rinv)*y

        if euclidean(y, y1) < eps:
            return y1

        y = y1

def corn_expert(self, data, w, c):
    '''
    :param w: window sze
    :param c: correlation coefficient threshold
    '''
    T, N = data.shape
    m = 0
    histdata = np.zeros((T,N))

    if T <= w+1:
        return np.ones(N) / N

    if w==0:
        histdata = data[:T,:]
        m = T
    else:
        for i in np.arange(w, T):
            d1 = data[i-w:i,:]
            d2 = data[T-w:T,:]
            datacorr = np.corrcoef(d1,d2)[0,1]

            if datacorr >= c:
                m += 1
                histdata[m,:] = data[i-1,:] #minus one to avoid out of bounds issue

    if m==0:
        return np.ones(N) / N

    #sqp according to OLPS implementation
    x_0 = np.ones((1,N)) / N
    objective = lambda b: -np.prod(np.dot(histdata, b))
    cons = ({'type': 'eq', 'fun': lambda b: 1-np.sum(b, axis=0)},)
    bnds = [(0.,1)]*N
    while True:
        res = minimize(objective, x_0, bounds=bnds, constraints=cons, method='slsqp')
        eps = 1e-7
        if (res.x < 0-eps).any() or (res.x > 1+eps).any():
            data += np.random.randn(1)[0] * 1e-5
            logging.debug('Optimal portfolio weight vector not found, trying again...')
            continue
        elif res.success:
            break
        else:
            if np.isnan(res.x).any():
                logging.warning('Solution does not exist, use uniform pwv')
                res.x = np.ones(N) / N
            else:
                logging.warning('Converged but not successfully.')
            break

    return res.x

def opt_weights(X, max_leverage=1):
    x_0 = max_leverage * np.ones(X.shape[1]) / np.float(X.shape[1])
    objective = lambda b: -np.prod(X.dot(b))
    cons = ({'type': 'eq', 'fun': lambda b: max_leverage-np.sum(b)},)
    bnds = [(0., max_leverage)]*len(x_0)
    while True:
        res = minimize(objective, x_0, bounds=bnds, constraints=cons, method='slsqp', options={'ftol':1e-07})
        eps = 1e-7
        if (res.x < 0-eps).any() or (res.x > max_leverage+eps).any():
            X = X + np.random.randn(1)[0] * 1e-5
            logging.debug('Optimal weights not found, trying again')
            continue
        elif res.success:
            break
        else:
            if np.isnan(res.x).any():
                logging.warning('Solution not found')
                res.x = np.ones(X.shape[1]) / X.shape[1]
            else:
                logging.warning("converged but not successfully")
            break

    return res.x


if __name__ == '__main__':
    data = load_mat('djia')
    n, m = data.shape
    print(data.shape)
    print(data[:n,:].shape)



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import ccxt
import pandas as pd
import numpy as np
import xarray as xr
#import sqlalchemy
from datetime import datetime, timedelta
#from mercurius.strategy import olmar


class CandleReader(object):
    """A wrapper for ccxt.

    1) Convert OHLCV to pandas dataframe
    2) Save dataframe to database (sqlite, sql, mongodb, etc.)
    """

    def __init__(self, symbols, start=(datetime.utcnow()-timedelta(minutes=60)).strftime("%Y-%m-%d %H:%M:%S"), end=(datetime.utcnow()).strftime("%Y-%m-%d %H:%M:%S"), timeframe='15m', exchange='poloniex'):
        """
        Args:
            symobols: a list of symbol of target assets
            start: (str) UTC start time
            end: (str) UTC end time
            timeframe (str): the interval getting data
            exchange (str): the exchange for downloading data
        """
        super(CandleReader, self).__init__()
        self.exchange = getattr(ccxt, exchange)()
        #print("start: ", start)
        #print("end: ", end)
        if not isinstance(start, int):
            self.start = self.exchange.parse8601(start)
        else:
            self.start = start
        #print("start: ", self.start)
        if not isinstance(end, int):
            self.end = self.exchange.parse8601(end)
        else:
            self.end = end
        self.timeframe = timeframe
        self.symbols = symbols

    def _to_df(self, ohlcv):
        #index = pd.Series(np.arange(start, end, interval), name='openTime')
        # init_chart = pd.DataFrame(np.nan, index=index, columns
        df = pd.DataFrame(
            ohlcv, columns=['timestamp', 'open', 'high', 'low', 'close', 'volume'])
        df['openTime'] = pd.to_datetime(df.timestamp / 1e3, unit='s')
        df.index = pd.DatetimeIndex(df['openTime']).round('S')
        df = df.drop(columns=['openTime', 'timestamp'])
        return df

    def get_data(self):
        raw_data_dict = {}
        if not isinstance(self.end, int) or not isinstance(self.start, int):
            raise TypeError("Instance Time Type needs to be int")

        if self.end > self.start:
            num_candles = (self.end-self.start) / \
                self.exchange.parse_timeframe(self.timeframe)/1e3
            #print("start time: ", self.start)
            #print("end time: ", self.end)
            #print("num of candles: ", num_candles)
        else:
            raise ValueError('end_date should be larger than start_data')

        #candle_list = []
        for symbol in self.symbols:
            ohlcv = self.exchange.fetch_ohlcv(
                symbol, self.timeframe, self.start, num_candles)
            #ohlcv = np.array(ohlcv)

            #features = ['open', 'high', 'low', 'close', 'volume']
            #timestamps = ohlcv[:,0]

            #print(ohlcv)
            #print(ohlcv.shape)
            #exit()
            raw_data_dict[symbol] = xr.DataArray(self._to_df(ohlcv), dims=['openTime', 'feature'])
            #arr = xr.DataArray(ohlcv, dims=['timestamp', 'open', 'high', 'low', 'close', 'volume'])
            #arr = self._to_df(ohlcv)
            #candle_list.append((symbol, arr))

        #chart = xr.concat(candle_list, dim=['asset', 'openTime', 'feature'])
        #chart = pd.MultiIndex.from_array(candle_list)
        chart = xr.Dataset(raw_data_dict).to_array(dim='asset')

        #chart = xr.DataArray(pd.Panel(raw_data_dict, dtype=np.float32), dims=[
        #                     "asset", "openTime", "feature"])
        #variables = {k: xr.DataArray()}
        #print(chart)
        #print(type(chart))
        return chart.transpose("feature", "asset", "openTime")

    def get_close(self, normalize=True, norm_rel=True):
        da = self.get_data()
        close_price = da.sel(feature='close')
        re = close_price.data.T
        btc_data = np.ones((re.shape[0],1))
        re = np.hstack((btc_data, re))

        if norm_rel:
            re = re[1:] / re[:-1]

        if normalize:
            re = re / re[0]

        return re

    def save_to_sql(self, df):
        df.to_csv('')
        pass
        #engine = create_engine("mysql+mysqldb://...")
        #df.to_sql('candlestick', engine, if_exists='replace', index=False, chunksize=10000)




from mercurius.strategy.base import expert
from mercurius.strategy import tools
import numpy as np
import logging

class best(expert):
    '''Benchmark '''
    def __init__(self, b=None):
        super(best, self).__init__()
        self.b = b

    def weights(self, data):
       n, m = data.shape
       tmp_cumprod_ret = np.cumprod(data, axis=0)
       best_ind = np.argmax(tmp_cumprod_ret[-1,:])
       weights = np.zeros((m,1))
       weights[best_ind] = 1
       return weights

    def get_b(self, data, last_b):
        pass

    def trade(self, data, tc=0, min_period=1):
        '''Meta Trader
        :param data:
        :param tc: transaction cost, default 0
        :param min_period: minimum period to start trading, default 1
        '''
        logging.info('------------------------------------------\n')
        logging.info('Parameters [tc: %f].\n' % tc)
        logging.info('day\t Daily Return\t Total Return\n')
        logging.info('------------------------------------------\n')

        n, m = data.shape


        cum_ret = 1 #cumulative return aka total return
        cumprod_ret = np.ones((n,1), np.float)
        daily_ret = np.ones((n, 1), np.float) #daily return

        b = np.ones((m,1), np.float) / m # initialize b
        re_b = np.zeros((m,1), np.float) # initialize rebalanced b
        daily_portfolio = np.zeros((n,m))

        b = self.weights(data)
        for t in range(n):

            # double check (Normalize the constraint)
            b = b / np.sum(b)
            daily_portfolio[t,:] = b.reshape((1,m))
            daily_ret[t] = np.dot(data[t,:],b)*(1-tc/2*np.sum(np.absolute(b-re_b)))
            cum_ret *= daily_ret[t]
            cumprod_ret[t] = cum_ret

            re_b = b * data[t,:][:,None] / daily_ret[t]

            logging.info('%d\t%f\t%f\n' % (t+1, daily_ret[t], cumprod_ret[t]))

        logging.info('tc=%f, Final Return: %.2f\n' % (tc, cum_ret))
        self.pDiff = daily_ret
        self.last_b = b

if __name__ == '__main__':
    tools.run(best())



from strategy import expert
import tools
import numpy as np


class pamr_old(expert):
    """
    OLPS implementation, exactly same reuslt as olps
    Passive aggressive mean reversion strategy for portfolio selection.
    There are three variants with different parameters, see original article
    for details.

    Reference:
        B. Li, P. Zhao, S. C.H. Hoi, and V. Gopalkrishnan.
        Pamr: Passive aggressive mean reversion strategy for portfolio selection, 2012.
        http://www.cais.ntu.edu.sg/~chhoi/paper_pdf/PAMR_ML_final.pdf
    """
    def __init__(self, eps=0.5, C=500, variant=0, b=None):
        """
        :param eps: Control parameter for variant 0. Must be >=0, recommended value is
                    between 0.5 and 1.
        :param C: Control parameter for variant 1 and 2. Recommended value is 500.
        :param variant: Variants 0, 1, 2 are available.
        """
        super(pamr_old, self).__init__()
        self.eps = eps
        self.C = C
        self.variant = variant
        self.b = b

    def get_b(self, data, last_b):
        if self.b is None:
            self.b = np.ones(data.shape[1]) / data.shape[1]

        # calculate return prediction
        b = self.update(self.b, data, self.eps, self.C)
        self.b = b
        return self.b


    def update(self, b, data, eps, C):
        """ Update portfolio weights to satisfy constraint b * x <= eps
        and minimize distance to previous weights. """
        T, N = data.shape
        x = data[-1,:]
        x_mean = np.mean(x)
        le = np.maximum(0., np.dot(b, x) - eps)
        denominator = np.square(np.linalg.norm(x - x_mean))

        if self.variant == 0:
            tau = le / denominator
        elif self.variant == 1:
            tau = np.minimum(C, le / denominator)
        elif self.variant == 2:
            tau = le / (denominator + 0.5 / C)

        # limit lambda to avoid numerical problems
        tau = np.minimum(100000, tau)

        # update portfolio
        b = b - tau * (data[T-1,:] - x_mean)
        # project it onto simplex
        return tools.simplex_proj(b.ravel())

if __name__ == "__main__":
    tools.run(pamr_old())



from mercurius.strategy.base import expert
from mercurius.strategy import tools
import numpy as np
import logging

class bnn(expert):
    '''
    Non-parametric
    '''
    def __init__(self, K=5, L=10, exp_w=None):
        super(bnn, self).__init__()
        self.K = K
        self.L = L
        self.exp_ret = np.ones((K,L+1))

    def get_b(self, data, last_b):
        n, m = data.shape

        if self.exp_w is None:
            self.exp_w = np.ones((K*(L+1),m)) / m

        self.exp_w[self.K*self.L,:] = self.update(data, 0, 0)

        for k in np.arange(self.K):
            for l in np.arange(self.L):
                pl = 0.02+0.5*(l-1)/(self.L-1)
                self.exp_w[(k-1)*self.L+l,:] = self.update(data, k, pl)

        p = 1./(self.K*self.L)
        numerator = p * self.exp_ret[0,self.L] * self.exp_w[self.K*self.L,:]
        denominator = p * self.exp_ret[0, self.L]

        for k in np.arange(self.K):
            for l in np.arange(self.L):
                numerator += p*self.exp_ret[k, l] * self.exp_w[(k-1)*self.L+l,:]
                denominator += p*self.exp_ret[k,l]

        weight = numerator.T / denominator

        return weight

    def update(self, data, k, l):
        T, N = data.shape
        m = 0
        histdata = np.zeros((T,N))

        if T <= k+1:
            return np.ones((1,N)) / N

        if k==0 and l==0:
            histdata = data[:T,:]
            m = T
        else:
            normid = np.zeros((T-k,1))
            histdata = data[:T,:]
            normid[:k] = 0
            for i in np.arange(k+1,T):
                data2 = data[i-k:i-1,:] - data[T-k+1:T,:]
                normid[:i] = np.sqrt(np.trace(data2.dot(data2.T)))
                sortpos = np.sort(normid)
                sortpos = sortpos.astype(int)
                m = int(np.floor(l*T))
                for j in np.arange(m):
                    histdata = np.vstack((histdata,histdata[int(sortpos[j]),:]))
        if m == 0:
            return np.ones((1,N)) / N

        b = tools.opt_weights(histdata)
        return b

    def trade(self, data, tc=0, min_period=1):
        '''Meta Trader
        :param data:
        :param tc: transaction cost, default 0
        :param min_period: minimum period to start trading, default 1
        '''
        logging.info('------------------------------------------\n')
        logging.info('Parameters [tc: %f].\n' % tc)
        logging.info('day\t Daily Return\t Total Return\n')
        logging.info('------------------------------------------\n')

        n, m = data.shape

        cum_ret = 1 #cumulative return aka total return
        cumprod_ret = np.ones((n,1), np.float)
        daily_ret = np.ones((n, 1), np.float) #daily return

        b = np.ones((m,1), np.float) / m # initialize b
        re_b = np.zeros((m,1), np.float) # initialize rebalanced b
        daily_portfolio = np.zeros((n,m))

        self.exp_ret = np.ones((self.K, self.L+1))
        self.exp_w = np.ones((self.K*(self.L+1), m)) / m

        for t in range(n):
            if t>= min_period:
                b = self.get_b(data[:t,:], re_b)

            # double check (Normalize the constraint)
            b = b / np.sum(b)
            daily_portfolio[t,:] = b.reshape((1,m))
            daily_ret[t] = np.dot(data[t,:],b)*(1-tc/2*np.sum(np.absolute(b-re_b)))
            cum_ret = cum_ret * daily_ret[t]
            cumprod_ret[t] = cum_ret

            re_b = b * data[t,:][:,None] / daily_ret[t]


            for k in np.arange(self.K):
                for l in np.arange(self.L):
                    self.exp_ret[k,l] *= np.dot(data[t,:],self.exp_w[(k-1)*self.L+l-1,:])

            logging.info('%d\t%f\t%f\n' % (t+1, daily_ret[t], cumprod_ret[t]))

        logging.info('tc=%f, Final Return: %.2f\n' % (tc, cum_ret))

if __name__ == "__main__":
    tools.run(bnn())



# -*- coding: utf-8 -*-
from mercurius.strategy.base import expert
from mercurius.strategy import tools
import numpy as np
import logging

class eg(expert):
    """ Exponentiated Gradient (EG) algorithm by Helmbold et al.

    Reference:
        Helmbold, David P., et al.
        "On‐Line Portfolio Selection Using Multiplicative Updates."
        Mathematical Finance 8.4 (1998): 325-347.
        http://www.cis.upenn.edu/~mkearns/finread/helmbold98line.pdf
    """

    def __init__(self, eta=0.05, b=None):
        """
        :params eta: Learning rate. Controls volatility of weights.
        """
        super(eg, self).__init__()
        self.eta = eta
        self.b = b

    def get_b(self, data, last_b):
        T, N = data.shape
        if self.b is None:
            self.b = np.ones((N,1)) / N
        last_b = self.b # the weight there is not rebalanced!
        self.b[:,0] = last_b[:,0] * np.exp(self.eta * data[T-1,:].T / (data[T-1,:].dot(last_b)))
        self.b = self.b/np.sum(self.b)
        return self.b

if __name__ == "__main__":
    tools.run(eg())



from mercurius.strategy import expert
import numpy as np
import logging
import tools

class anticort(expert):
    '''test
    anti-correlation olps
    '''
    def __init__(self, window=30):
        super(anticort, self).__init__()
        self.window = window

    def get_b(self, data, last_b):
        for k in np.arange(1,self.window):
            self.exp_w[k-1,:] = self.update(data, self.exp_w[k-1,:].T, k)

        numerator = 0
        denominator = 0

        for k in np.arange(1,self.window):
            numerator += self.exp_ret[k-1, 0] * self.exp_w[k-1,:]
            denominator += self.exp_ret[k-1,0]

        weight = numerator.T / denominator

        return weight

    def update(self, data,last_b, w):
        T, N = data.shape

        b = last_b
        if T >= 2*w:
            LX1 = np.log(data[T-2*w+1:T-w,:]) #(k,N)
            LX2 = np.log(data[T-w+1:T,:])

            mu1 = np.mean(LX1, axis=0)
            mu2 = np.mean(LX2, axis=0)

            n_LX1 = np.subtract(LX1, mu1)
            n_LX2 = np.subtract(LX2, mu2)

            std1 = np.std(n_LX1, axis=1)
            std2 = np.std(n_LX2, axis=1)
            std12 = np.dot(std1.T, std2) # sigma

            mu_matrix = np.ones((mu2.shape[0],mu2.shape[0]), dtype = bool)

            for i in range(N):
                for j in range(N):
                    if mu2[i] > mu2[j]:
                        mu_matrix[i,j] = True
                    else:
                        mu_matrix[i,j] = False

            mCv = np.zeros((N,N))
            mCorr = np.zeros((N,N))
            mCov = np.dot(n_LX1.T, n_LX2) / np.float64(w-1)

            mCorr = np.where(std12!=0, np.divide(mCov,std12), std12) #set

            s12 =  np.multiply(mCorr>0,mu_matrix)
            claim = np.zeros((N,N))
            claim[s12] += mCorr[s12]


            bool_claim = np.absolute(claim) > 0

            diag_mCorr = np.diag(mCorr)
            cor1 = np.maximum(0, np.tile(-diag_mCorr, (1,N))).reshape((N,N))
            cor2 = np.maximum(0, np.tile(-diag_mCorr.T, (N,1)))
            claim[s12] += (cor1[s12] + cor2[s12])

            transfer = np.zeros((N,N))

            sum_claim = np.tile(np.sum(claim, axis=1), (N,1))

            s1 = np.absolute(sum_claim) > 0
            w_b = np.tile(b, (N,1))
            transfer[s1] = w_b[s1] * claim[s1] / sum_claim[s1]

            transfer = np.where(np.isnan(transfer), 0, transfer)
            transfer_ij = transfer.T - transfer
            #print np.sum(transfer_ij)
            b -= np.sum(transfer_ij)
        return b

    def trade(self, data, tc=0, min_period=1):
        '''Meta Trader
        :param data:
        :param tc: transaction cost, default 0
        :param min_period: minimum period to start trading, default 1
        '''
        logging.info('------------------------------------------\n')
        logging.info('Parameters [tc: %f].\n' % tc)
        logging.info('day\t Daily Return\t Total Return\n')
        logging.info('------------------------------------------\n')

        n, m = data.shape

        cum_ret = 1 #cumulative return aka total return
        cumprod_ret = np.ones((n,1), np.float)
        daily_ret = np.ones((n, 1), np.float) #daily return

        b = np.ones((m,1), np.float) / m # initialize b
        re_b = np.zeros((m,1), np.float) # initialize rebalanced b
        daily_portfolio = np.zeros((n,m))

        self.exp_ret = np.ones((self.window-1,1))
        self.exp_w = np.ones((self.window-1,m)) / m

        for t in range(n):
            if t>= min_period:
                b = self.get_b(data[:t,:], re_b)

            # double check (Normalize the constraint)
            b = b / np.sum(b)
            daily_portfolio[t,:] = b.reshape((1,m))
            daily_ret[t] = np.dot(data[t,:],b)*(1-tc/2*np.sum(np.absolute(b-re_b)))
            cum_ret = cum_ret * daily_ret[t]
            cumprod_ret[t] = cum_ret

            re_b = b * data[t,:][:,None] / daily_ret[t]

            for k in np.arange(1,self.window):
                self.exp_ret[k-1,0] = np.dot(self.exp_ret[k-1,0] * data[k,:],self.exp_w[k-1,:].T)
            self.exp_ret[:,0] /= np.sum(self.exp_ret[:,0])

            logging.info('%d\t%f\t%f\n' % (t+1, daily_ret[t], cumprod_ret[t]))

        logging.info('tc=%f, Final Return: %.2f\n' % (tc, cum_ret))

if __name__ == "__main__":
    tools.run(anticort())



from mercurius.strategy.base import expert
from mercurius.strategy import tools
import numpy as np
import logging

class rmr(expert):
    ''' Robust Median Reversion

    Reference:


    '''
    def __init__(self, eps=5, W=5, b=None):
        '''
        :param eps: the parameter control the reversion threshold
        :pram W: the length of window
        '''
        super(rmr, self).__init__()
        self.eps = eps
        self.W = W
        self.b = b

    def get_b(self, data, last_b):
        pass

    def update(self, data_close, data, t1,  last_b, eps, W):
        #print data.shape
        T, N = data.shape
        if self.b is None:
            self.b = np.ones(N) / N

        last_b = self.b
        if t1 < W+2:
            x_t1 = data[t1-2, :]
        else:
            x_t1 = tools.l1_median_VaZh(data_close[(t1-W-1):(t1-2),:]) / data_close[(t1-2),:]

        if np.linalg.norm(x_t1 - x_t1.mean())**2 == 0:
            tao = 0
        else:
            tao = np.minimum(0, (x_t1.dot(last_b)-eps) / np.linalg.norm(x_t1 - x_t1.mean())**2)
        self.b -= tao * (x_t1 - np.mean(x_t1) * np.ones(x_t1.shape))
        self.b = tools.simplex_proj(self.b)
        return self.b

    def get_close(self, data):
        return np.cumprod(data,axis=0)

    def trade(self, data, tc=0, min_period=1):
        '''Meta Trader
mr_var:param data:
        :param tc: transaction cost, default 0
        :param min_period: minimum period to start trading, default 1
        '''
        logging.info('------------------------------------------\n')
        logging.info('Parameters [tc: %f].\n' % tc)
        logging.info('day\t Daily Return\t Total Return\n')
        logging.info('------------------------------------------\n')

        n, m = data.shape

        data_close = self.get_close(data)

        cum_ret = 1 #cumulative return aka total return
        cumprod_ret = np.ones((n,1), np.float)
        daily_ret = np.ones((n, 1), np.float) #daily return

        b = np.ones((m,1), np.float) / m # initialize b
        re_b = np.zeros((m,1), np.float) # initialize rebalanced b
        daily_portfolio = np.zeros((n,m))

        for t in range(n):

            b = self.update(data_close, data, t+1, self.b, self.eps, self.W)
            daily_portfolio[t,:] = b.reshape((1,m))
            daily_ret[t] = np.dot(data[t,:],b)*(1-tc/2*np.sum(np.absolute(b-re_b)))
            cum_ret *= daily_ret[t]
            cumprod_ret[t] = cum_ret
            re_b = b * data[t,:][:,None] / daily_ret[t]

            logging.info('%d\t%f\t%f\n' % (t+1, daily_ret[t], cumprod_ret[t]))

        logging.info('tc=%f, Final Return: %.2f\n' % (tc, cum_ret))

        self.pDiff = daily_ret

if __name__ == "__main__":
    tools.run(rmr())



from __future__ import absolute_import
from __future__ import print_function
from __future__ import division

from flask import Flask, request, json
from flask_cors import CORS, cross_origin
from mercurius.data import candlereader
from datetime import datetime, timedelta
import numpy as np
import ccxt
from mercurius.strategy.ons import ons
from mercurius.strategy.ucrp import ucrp
from mercurius.strategy.ubah import ubah
from mercurius.strategy.best import best
from mercurius.data.candlereader import CandleReader
from mercurius.utils.indicators import max_drawdown, sharpe, positive_count, negative_count, moving_accumulate, sortino
import requests

app = Flask(__name__)
cors = CORS(app)
app.config['CORS_HEADERS'] = 'Content-Type'

ALGOS = {'ucrp': ucrp, 'ons': ons, 'ubah': ubah, 'best': best}
INDICATORS = {"portfolio value": np.prod,
              "sharpe ratio": sharpe,
              "max drawdown": max_drawdown,
              "positive periods": positive_count,
              "negative periods": negative_count,
              "sortino ratio": sortino,
              "postive day": lambda pcs: positive_count(moving_accumulate(pcs, 48)),
              "negative day": lambda pcs: negative_count(moving_accumulate(pcs, 48)),
              "postive week": lambda pcs: positive_count(moving_accumulate(pcs, 336)),
              "negative week": lambda pcs: negative_count(moving_accumulate(pcs, 336)),
              "average": np.mean}

@app.route('/uploadAlgo', methods=['POST', 'GET'])
def run_algo():
    if request.method == "POST":
        data = request.get_json()
        exchange = data["exchange"]
        start_time = data["start_time"]
        end_time = data["end_time"]
        timeframe = data["timeframe"]
        symbols = data["symbols"]
        da = candlereader.CandleReader(
            symbols, start_time, end_time, timeframe, exchange).get_data()
        close_price = da.sel(feature='close')
        # algo.run()
        response = {}
        response['close'] = close_price.data.tolist()

        response['date'] = [
            x / 1e6 for x in da['openTime'].values.astype(np.int64)]
        # print(response['date'])
        # print(type(response['date']))
        return json.jsonify(response)
    return 'FAIL'


@app.route('/coin', methods=['POST', 'GET'])
@cross_origin()
def get_coin_price(exchange="poloniex"):
    """default poloniex"""
    res = {}
    symbol = request.args.get('symbol', 'ETH')
    start = request.args.get('start', (datetime.utcnow(
    ) - timedelta(days=30)).strftime("%Y-%m-%d %H:%M:%S"))
    end = request.args.get(
        'end', (datetime.utcnow()).strftime("%Y-%m-%d %H:%M:%S"))
    algo = request.args.get('algo', 'ucrp')

    # fixed trading list
    trading_list = ['ETH/BTC', 'ETC/BTC', 'LTC/BTC', 'XRP/BTC', 'XLM/BTC', 'XEM/BTC']

    tf = request.args.get('tf', '30m')  # timeframe
    #exchange = getattr(ccxt, exchange)
    exchange = ccxt.poloniex()
    pair = str(symbol).upper() + "/BTC"
    start = exchange.parse8601(start)
    end = exchange.parse8601(end)
    num_candles = int((end - start) / exchange.parse_timeframe(tf) / 1e3)
    data = exchange.fetch_ohlcv(pair, tf, start, num_candles)
    time_list = []
    ohlc = []
    indexes = [1,4,2,3]
    volume = []
    counter = 0
    for i in data:
        time_list.append((datetime.utcfromtimestamp(int(i[0] / 1e3))).strftime('%Y-%m-%d %H:%M:%S'))
        ohlc.append([i[k] for k in indexes])
        bool_vol = 1 if i[1] > i[4] else -1
        volume.append([counter, i[5], bool_vol])
        counter += 1
    #app.logger.info(data)

    close_data = CandleReader(trading_list, start, end, tf).get_close(False, True)
    agent = ALGOS[algo]()
    agent.trade(close_data, tc=0.025)
    portfolio_value = agent.finish()
    m = close_data.shape[1]

    ind_re = []
    for ind in list(INDICATORS.keys()):
        rs = INDICATORS[ind](portfolio_value['portfolio_diff'][:-1])
        if isinstance(rs, np.ndarray):
            #print(ind, rs)
            rs = rs.tolist()[0]
        if isinstance(rs, np.int64):
            #print(ind, rs)
            rs = int(rs)
        if np.isnan(rs):
            rs = 0
        ind_re.append(rs)

    res['time'] = time_list
    res['ochl'] = ohlc
    res['vol'] = volume
    res['symbols'] = ['BTC'] + trading_list
    res['pv'] = np.nan_to_num(portfolio_value['portfolio']).tolist() #do not get the last one(NaN)
    res['ind'] = ind_re
    res['pw'] = portfolio_value['last_b'].T.tolist()

    return json.jsonify(res)

# NOTE: For now, (Buggy) need a time stamp for a start exactly starting from a day
@app.route('/backtest/<starttimestamp>/<endtimestamp>', methods=['GET'])
def get_back_test(starttimestamp, endtimestamp):
    res = backtest(
        datetime.fromtimestamp(int(starttimestamp)).strftime("%Y-%m-%d %H:%M:%S"),
        datetime.fromtimestamp(int(endtimestamp)).strftime("%Y-%m-%d %H:%M:%S"),
        location="../../train_package/"
    )
    # res = backtest("2018-06-02 00:00:00", "2018-06-06 00:00:00", location="../../train_package/")
    time_values = res.get("portfolio_changes_history").keys().tolist()
    rate_values = [value for value in generator(res.get("portfolio_changes_history").values)]
    return json.jsonify({"xaxis": time_values, "data": rate_values})

@app.route('/vollist', methods=['POST', 'GET'])
@cross_origin()
def sort_by_vol():
    re = requests.get('https://pro-api.coinmarketcap.com/v1/cryptocurrency/listings/latest?limit=20', headers={'X-CMC_PRO_API_KEY': 'f22cbc37-a098-4fb3-84f4-872f38d2b023'})
    return re.content


def generator(list):
    res = 1
    for i in list:
        yield i
        res *= i

def main():
    app.run(host='0.0.0.0', debug=True)

if __name__ == "__main__":
    import sys, os
    sys.path.append(os.getcwd())
    main()



from __future__ import division, print_function, absolute_import
import numpy as np


def max_drawdown(pc_array):
    """calculate the max drawdown with the portfolio changes
    @:param pc_array: all the portfolio changes during a trading process
    @:return: max drawdown
    """
    portfolio_values = []
    drawdown_list = []
    max_benefit = 0
    for i in range(pc_array.shape[0]):
        if i > 0:
            portfolio_values.append(portfolio_values[i - 1] * pc_array[i])
        else:
            portfolio_values.append(pc_array[i])
        if portfolio_values[i] > max_benefit:
            max_benefit = portfolio_values[i]
            drawdown_list.append(0.0)
        else:
            drawdown_list.append(1.0 - portfolio_values[i] / max_benefit)
    return max(drawdown_list)


def sharpe(pc_array):
    """calculate sharpe ratio with the portfolio changes
    @:param pc_array: all the portfolio changes during a trading process
    @:return: sharpe ratio
    """
    pc_array = pc_array-1.0
    return np.mean(pc_array)/np.std(pc_array)

def sortino(pc_array):
    returns = pc_array-1.0
    downside_risk = (np.mean(returns[returns<0]**2))**(1/2)
    if downside_risk == 0 or downside_risk == np.nan:
        return 0 if np.sum(returns) == 0 else np.inf
    elif abs(np.mean(returns)) < 1e-5:
        return 0
    return np.mean(returns)/downside_risk

def factored_sortino(pc_array, n=0.25):
    returns = pc_array-1.0
    downside_risk = (np.mean(returns[returns<0]**2))**(1/2)
    if downside_risk == 0 or downside_risk == np.nan:
        return 0 if np.sum(returns) == 0 else np.inf
    elif abs(np.mean(returns)) < 1e-5:
        return 0
    return np.mean(returns)/np.power(downside_risk, n)

def cash_risk_sortino(pc_array, cash_risk=0.04):
    """
    sortino ratio that involves the rsik of cash
    :param pc_array:
    :param cash_risk: estimated cash risk
    :return:
    """
    returns = pc_array - 1.0
    downside_risk = (np.mean(returns[returns < 0] ** 2)) ** (1 / 2)
    if downside_risk == 0 or downside_risk == np.nan:
        return 0 if np.sum(returns) == 0 else np.inf
    elif abs(np.mean(returns)) < 1e-5:
        return 0
    return np.mean(returns) / (downside_risk+cash_risk)

def moving_accumulate(pc_array, n=48):
    acc = np.cumprod(pc_array)
    acc[n:] = acc[n:] / acc[:-n]
    return acc


def positive_count(pc_array):
    return np.sum(pc_array>1)


def negative_count(pc_array):
    return np.sum(pc_array<1)



from mercurius.strategy.base import expert
from mercurius.strategy import tools
import numpy as np
from cvxopt import solvers, matrix
solvers.options['show_progress'] = False


class ons(expert):
    """
    Online newton step algorithm.

    Reference:
        A.Agarwal, E.Hazan, S.Kale, R.E.Schapire.
        Algorithms for Portfolio Management based on the Newton Method, 2006.
        http://machinelearning.wustl.edu/mlpapers/paper_files/icml2006_AgarwalHKS06.pdf
    """
    def __init__(self, delta=0.125, beta=1., eta=0., A=None, b=None):
        """
        :param delta, beta, eta: Model parameters. See paper.
        """
        super(ons, self).__init__()
        self.delta = delta
        self.beta = beta
        self.eta = eta
        self.A = A
        self.b = b

    def init_portfolio(self, X):
        m = X.shape[1]
        self.A = np.mat(np.eye(m))
        self.bt = np.mat(np.zeros(m)).T


    def get_b(self, data, last_b):
        '''
        :param x: input matrix
        :param last_b: last portfolio
        '''
        T, N = data.shape
        if self.b is None:
            self.b = np.ones(N) / N

        last_b = self.b # don't rebalance last_b

        if self.A is None:
            self.init_portfolio(data)

        # calculate gradient
        grad = np.mat(data[-1,:] / np.dot(last_b,data[-1,:])).T
        # update A
        self.A += grad * grad.T
        # update b
        self.bt += (1 + 1./self.beta) * grad

        # projection of p induced by norm A
        pp = self.projection_in_norm(self.delta * self.A.I * self.bt, self.A)
        self.b = pp * (1 - self.eta) + np.ones(N)/N * self.eta
        return self.b


    def projection_in_norm(self, x, M):
        """ Projection of x to simplex indiced by matrix M. Uses quadratic programming.
        """
        m = M.shape[0]

        P = matrix(2*M)
        q = matrix(-2 * M * x)
        G = matrix(-np.eye(m))
        h = matrix(np.zeros((m,1)))
        A = matrix(np.ones((1,m)))
        b = matrix(1.)

        sol = solvers.qp(P, q, G, h, A, b)
        return np.squeeze(sol['x'])

if __name__ == "__main__":
    tools.run(ons())



from mercurius.strategy.base import expert
from mercurius.strategy import tools
import numpy as np
import logging

class ucrp(expert):
    """uniform constant rebalanced portfolio"""
    def __init__(self):
        super(ucrp, self).__init__()

    def get_b(self, data, last_b):
        n, m = data.shape
        return np.ones((m,1))/m

if __name__ == '__main__':
    #logging.basicConfig(level=logging.INFO)
    #data = tools.load_mat('djia')
    #test = ucrp()
    #test.trade(data)
    tools.run(ucrp())



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('requirements.txt') as f:
    reqs = f.read().splitlines()

setuptools.setup(
    name="mercurius",
    version="0.0.1",
    author="Dixing (Dex) Xu",
    author_email="dixingxu@gmail.com",
    description="Yet Another Portfolio Management Toolbox",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/dexhunter/mercurius-dev",
    packages=setuptools.find_packages(),
    install_requires=reqs,
)



from mercurius.strategy.base import expert
from mercurius.strategy import tools
import numpy as np
import logging
from scipy.optimize import minimize

class bcrp(expert):
    """ Best Constant Rebalanced Portfolio = Constant Rebalanced Portfolio constructed with hindsight. It is often used as benchmark.

    Reference:
        T. Cover. Universal Portfolios, 1991.
        http://www-isl.stanford.edu/~cover/papers/paper93.pdf
    """

    def __init__(self, b=None):
        super(bcrp, self).__init__()
        self.b = b

    def weights(self, data):
        """ Find weights which maximize return on X in hindsight! """
        return opt_weights(data)

    def get_b(self, data, last_b):
        pass

    def trade(self, data, tc=0, min_period=1):
        '''Meta Trader
        :param data:
        :param tc: transaction cost, default 0
        :param min_period: minimum period to start trading, default 1
        '''
        logging.info('------------------------------------------\n')
        logging.info('Parameters [tc: %f].\n' % tc)
        logging.info('day\t Daily Return\t Total Return\n')
        logging.info('------------------------------------------\n')

        n, m = data.shape


        cum_ret = 1 #cumulative return aka total return
        cumprod_ret = np.ones((n,1), np.float)
        daily_ret = np.ones((n, 1), np.float) #daily return

        b = np.ones((m,1), np.float) / m # initialize b
        re_b = np.zeros((m,1), np.float) # initialize rebalanced b
        daily_portfolio = np.zeros((n,m))

        b = self.weights(data)
        for t in range(n):

            # double check (Normalize the constraint)
            b = b / np.sum(b)
            daily_portfolio[t,:] = b.reshape((1,m))
            daily_ret[t] = np.dot(data[t,:],b)*(1-tc/2*np.sum(np.absolute(b-re_b)))
            cum_ret *= daily_ret[t]
            cumprod_ret[t] = cum_ret

            re_b = b * data[t,:][:,None] / daily_ret[t]

            logging.info('%d\t%f\t%f\n' % (t+1, daily_ret[t], cumprod_ret[t]))

        logging.info('tc=%f, Final Return: %.2f\n' % (tc, cum_ret))
        self.pDiff = daily_ret
        self.last_b = b

def opt_weights(X, max_leverage=1):
    x_0 = max_leverage * np.ones(X.shape[1]) / float(X.shape[1])
    objective = lambda b: -np.prod(X.dot(b))
    cons = ({'type': 'eq', 'fun': lambda b: max_leverage-np.sum(b)},)
    bnds = [(0., max_leverage)]*len(x_0)
    res = minimize(objective, x_0, bounds=bnds, constraints=cons, method='slsqp', options={'ftol': 1e-07})
    return res.x


if __name__ == '__main__':
    tools.run(bcrp())



from mercurius.strategy.pamr import pamr
from mercurius.strategy import tools
import numpy as np


class wmamr(pamr):
    """ Weighted Moving Average Passive Aggressive Algorithm for Online Portfolio Selection.
    It is just a combination of OLMAR and PAMR, where we use mean of past returns to predict
    next day's return.

    Reference:
        Li Gao, Weiguo Zhang
        Weighted Moving Averag Passive Aggressive Algorithm for Online Portfolio Selection, 2013.
        http://ieeexplore.ieee.org/xpl/login.jsp?tp=&arnumber=6643896
    """

    def __init__(self, window=5):
        """
        :param w: Windows length for moving average.
        """
        super(wmamr, self).__init__()

        if window < 1:
            raise ValueError('window parameter must be >=1')
        self.window = window


    def get_b(self, x, last_b):
        self.record_history(x)
        xx = np.mean(self.history[-self.window:,], axis=0)
        # calculate return prediction
        b = self.update(last_b, xx, self.eps, self.C)

        return b




from mercurius.strategy.base import expert
from mercurius.strategy import tools
import numpy as np

class olmar2(expert):
    '''Moving average reversion strategy for on-line portfolio selection

    Reference:
        Bin Li, Steven C.H. Hoi, Doyen Sahoo, Zhi-Yong Liu

    '''

    def __init__(self,  eps=10, alpha=0.5):
        '''init
        :param eps: mean reversion threshold
        :param alpha: trade off parameter for moving average
        '''
        super(olmar2, self).__init__()
        self.eps = eps
        self.alpha = alpha


    def get_b(self, data, last_b):
        T, N = data.shape
        data_phi = np.ones((1,N))
        data_phi = self.alpha + (1-self.alpha)*data_phi/data[T-1,:]


        ell = max(0, self.eps - data_phi.dot(last_b))

        x_bar = data_phi.mean()
        denominator = np.linalg.norm(data_phi - x_bar)**2

        if denominator == 0:
            lam = 0
        else:
            lam = ell / denominator

        data_phi = np.squeeze(data_phi)
        b = last_b.ravel() +  lam * (data_phi - x_bar)
        b = tools.simplex_proj(b.ravel())
        return b[:,None]

if __name__ == "__main__":
    tools.run(olmar2())
